function [ctf] = ctf_read_res4(folder,VERBOSE);

% ctf_read_res4 - Read a CTF .res4 file
%
% ctf = ctf_read_res4( [folder], [VERBOSE])
% 
% This function reads the resource information from a CTF .ds folder.  This
% resource information must be read before reading the data from a .meg4
% file.
%
% INPUTS
%
%   folder: the .ds directory containing the data to be read.  With
%   no input, it will prompt with a gui folder locator.  If only
%   the folder is input, VERBOSE = 1.
%
%   VERBOSE = 1, display 'ctf.res4.generalSetup' structure (default)
%   VERBOSE = 0, do not display 'ctf.res4.generalSetup' structure
%
% OUTPUTS
%
%   ctf.folder: path of the .ds folder
%
%   ctf.header: data format header
%
%   ctf.res4.generalSetup: a header structure consisting of date, time, run name, run
%   title, subject, run description, operator, number of channels, number
%   of samples, sample rate, number of trials, duration, pretriggerSamples,
%   sensor filename, head zeroing, and number of filters.
%   
%   ctf.sensor.index: a sensor structure consisting of EEG sensors, MEG
%   sensors, reference sensors, and other sensors.
%   
%   ctf.sensor.info: a structure with gain and offset information consisting
%   of proper gain, Q gain, io gain, io offset, and index.
%
%      <>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
%      <                                                      >
%      <                    DISCLAIMER:                       >
%      <                                                      >
%      < THIS PROGRAM IS INTENDED FOR RESEARCH PURPOSES ONLY. >
%      < THIS PROGRAM IS IN NO WAY INTENDED FOR CLINICAL OR   >
%      <                    OFFICIAL USE.                     >
%      <                                                      >
%      <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>
%

% $Revision: 1.2 $ $Date: 2004/02/02 23:57:40 $

% Licence:  GNU GPL, no express or implied warranties
% Modified: 11/2003, Darren.Weber_at_radiology.ucsf.edu
%                    - modified from NIH code readresfile.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ver = '$Revision: 1.2 $';
fprintf('\nCTF_READ_RES4 [v %s]\n',ver(11:15)); tic;

if ~exist('folder','var'),
  ctf = ctf_folder;
else
  ctf = ctf_folder(folder);
end

if ~exist('VERBOSE','var'), VERBOSE = 1; end

[rootpath,rootname] = fileparts(ctf.folder);
res4file = [rootname,'.res4'];
[res4fid,message] = fopen(res4file,'rb','s');

% 'ieee-be.l64' or 's' - IEEE floating point with big-endian byte ordering
%                        and 64 bit long data type.

if res4fid < 0,
  error(message);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% READ HEADER

fseek(res4fid, 0,-1);
ctf.res4.header = char(fread(res4fid,8,'char'))';

% check for the right format
if strmatch('MEG41RS',ctf.res4.header),
  % OK, we can handle this format
else
  msg = sprintf('May not read "%s" format correctly',ctf.res4.header);
  warning(msg);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% READ GENERAL RESOURCES, byte offset = 8, byte size = 1840


%---DATE/TIME
fseek(res4fid, 778,-1);
ctf.res4.generalSetup.time = char(fread(res4fid,255,'char'))';
fseek(res4fid,1033,-1);
ctf.res4.generalSetup.date = char(fread(res4fid,255,'char'))';

%---NUMBER OF SAMPLES
fseek(res4fid,1288,-1);
ctf.res4.generalSetup.numberSamples = (fread(res4fid,1,'int32')');

%---NUMBER OF CHANNELS
fseek(res4fid,1292,-1);
ctf.res4.generalSetup.numberChannels = (fread(res4fid,1,'int16')');

%---SAMPLE RATE
fseek(res4fid,1296,-1);
ctf.res4.generalSetup.sampleRate = fread(res4fid,1,'double')';
ctf.res4.generalSetup.sampleMsec = 1000 / ctf.res4.generalSetup.sampleRate;
ctf.res4.generalSetup.sample_sec  =    1 / ctf.res4.generalSetup.sampleRate;

%---NUMBER OF TRIALS
fseek(res4fid,1312,-1);
ctf.res4.generalSetup.numberTrials = fread(res4fid,1,'int16')';
fseek(res4fid, 776,-1);
ctf.res4.generalSetup.numberTrials_averaged = fread(res4fid,1,'int16');

%---DURATION
fseek(res4fid,1304,-1);
ctf.res4.generalSetup.durationTotal = fread(res4fid,1,'double');
ctf.res4.generalSetup.durationTrial = ctf.res4.generalSetup.durationTotal / ctf.res4.generalSetup.numberTrials;

%---PRE_TRIG POINTS
fseek(res4fid,1316,-1);
ctf.res4.generalSetup.pretriggerSamples = fread(res4fid,1,'int32');
ctf.res4.generalSetup.pretriggerMsec = (ctf.res4.generalSetup.pretriggerSamples / ctf.res4.generalSetup.sampleRate) * 1000;

%---HEAD ZEROING
fseek(res4fid,1348,-1);
h_zero = fread(res4fid,1,'int32')';
no_yes = {'no','yes'};
ctf.res4.generalSetup.headZero = no_yes{h_zero+1};

%---RUN NAME
fseek(res4fid,1360,-1);
ctf.res4.generalSetup.runName = char(fread(res4fid,32,'char'))';

%---RUN TITLE
fseek(res4fid,1392,-1);
ctf.res4.generalSetup.runTitle = char(fread(res4fid,256,'char'))';

%---SUBJECT
fseek(res4fid,1712,-1);
ctf.res4.generalSetup.subject = char(fread(res4fid,32,'char'))';

%---OPERATOR
fseek(res4fid,1744,-1);
ctf.res4.generalSetup.operator = char(fread(res4fid,32,'char'))';

%---SENSOR FILE NAME
fseek(res4fid,1776,-1);
ctf.res4.generalSetup.sensorFileName = char(fread(res4fid,60,'char'))';


%---CALCULATE TIME ARRAYS (not read directly from the .res4 file)

% the time arrays must be based on increments of the sampleMsec
ctf.res4.generalSetup.timeMsec = [0:ctf.res4.generalSetup.numberSamples - 1]' * ctf.res4.generalSetup.sampleMsec;
ctf.res4.generalSetup.timeMsec = ctf.res4.generalSetup.timeMsec - ctf.res4.generalSetup.pretriggerMsec;

% adjust the sample point closest to zero so that it is zero, if it
% is reasonably close to zero, say within 3 decimal places for msec timing
zeroIndex = find(abs(ctf.res4.generalSetup.timeMsec) == min(abs(ctf.res4.generalSetup.timeMsec)));
zeroValue = ctf.res4.generalSetup.timeMsec(zeroIndex);
if (-0.0001 < zeroValue) & (zeroValue < 0.0001),
  ctf.res4.generalSetup.timeMsec(zeroIndex) = 0;
end

ctf.res4.generalSetup.startMsec = ctf.res4.generalSetup.timeMsec(1);
ctf.res4.generalSetup.endMsec   = ctf.res4.generalSetup.timeMsec(end);

ctf.res4.generalSetup.timeSec  = ctf.res4.generalSetup.timeMsec / 1000;
ctf.res4.generalSetup.startSec = ctf.res4.generalSetup.timeSec(1);
ctf.res4.generalSetup.endSec   = ctf.res4.generalSetup.timeSec(end);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% UNUSED, byte offset = 1840, byte size = 4

fseek(res4fid,1840,-1);
unused = fread(res4fid,1,'int32'); % read 4 byte int32 value

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RUN DESCRIPTION, byte offset = 1844, byte size = 1 x rdl

% run description length (rdl)
fseek(res4fid,1836,-1);
runDescriptionLength = fread(res4fid,1,'int32');
% run description
fseek(res4fid,1844,-1);
ctf.res4.runDescription = char(fread(res4fid,runDescriptionLength,'char'));


%---PRINT RES4 VERBOSE INFORMATION
if VERBOSE,
  fprintf('...data format       : %s\n',ctf.res4.header);
  fprintf('...date collected    : %s\n',ctf.res4.generalSetup.date);
  fprintf('...time collected    : %s\n',ctf.res4.generalSetup.time);
  fprintf('...run name          : %s\n',ctf.res4.generalSetup.runName);
  fprintf('...run title         : %s\n',ctf.res4.generalSetup.runTitle);
  fprintf('...subject           : %s\n',ctf.res4.generalSetup.subject);
  fprintf('...run description   : %s\n',ctf.res4.runDescription);
  fprintf('...operator          : %s\n',ctf.res4.generalSetup.operator);
  fprintf('...number of channels: %d\n',ctf.res4.generalSetup.numberChannels);
  fprintf('...number of samples : %d per trial\n',ctf.res4.generalSetup.numberSamples);
  fprintf('...sample rate       : %g samples/sec\n',ctf.res4.generalSetup.sampleRate);
  fprintf('...number of trials  : %g trials (average of %g)\n',ctf.res4.generalSetup.numberTrials,ctf.res4.generalSetup.numberTrials_averaged);
  fprintf('...duration          : %g sec total\n',ctf.res4.generalSetup.durationTotal);
  fprintf('...duration          : %g sec / trial\n',ctf.res4.generalSetup.durationTrial);
  fprintf('...pretrigger points : %g samples\n',ctf.res4.generalSetup.pretriggerSamples);
  fprintf('...pretrigger msec   : %g msec\n',ctf.res4.generalSetup.pretriggerMsec);
  fprintf('...sensor file name  : %s\n',ctf.res4.generalSetup.sensorFileName);
  fprintf('...head zeroing      : %s\n',ctf.res4.generalSetup.headZero);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NUMBER OF FILTERS, byte offset = 1844 + rdl, byte size = 2

offset = 1844 + runDescriptionLength;
fseek(res4fid,offset,-1);
ctf.res4.numberFilters = fread(res4fid,1,'int16');

% this is 'nf' in CTF file format pdf, used for calculating file byte offset
nf = ctf.res4.numberFilters;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FILTER INFORMATION, byte offset = 1846 + rdl, byte size = variable

% this is 'b' in CTF file format pdf, used for calculating file byte offset
b = 1846 + runDescriptionLength;
fseek(res4fid,b,-1);


% this is 'fi' in CTF file format pdf, used for calculating file byte offset
fi = 0;
% this is 'p' & 'np' in CTF file format pdf
p = 0; np = 0;

for i = 1:ctf.res4.numberFilters,
  
  fi = i - 1;
  ctf.res4.filter(i).frequency = fread(res4fid,1,'double');
  ctf.res4.filter(i).class     = fread(res4fid,1,'int32');
  ctf.res4.filter(i).type      = fread(res4fid,1,'int32');
  
  if i > 1,
    p = p + ( ctf.res4.filter(i-1).numberParameters * 8 );
  end
  np = np + ( ctf.res4.filter(i).numberParameters * 8 );
  
  ctf.res4.filter(i).numberParameters = fread(res4fid,1,'int16');
  ctf.res4.filter(i).parameters = fread(res4fid,ctf.res4.filter(i).numberParameters,'double');
  
end

% this is 'f' in CTF file format pdf
f = ( nf * 18 ) + ( np * 8 );


%---PRINT FILTER INFORMATION
if VERBOSE,
  fprintf('...number of filters : %d\n',ctf.res4.numberFilters);
  for i = 1:ctf.res4.numberFilters,
    fprintf(' -Filter # \t%g\n',i)
    fprintf(' -Frequency: \t%g Hz\n',ctf.res4.filter(i).frequency)
    fprintf(' -Class: \t%g\n',ctf.res4.filter(i).class)
    fprintf(' -Type: \t%g\n',ctf.res4.filter(i).type)
    if ~isempty(ctf.res4.filter(i).parameters)
      fprintf(' -Parameter(s): \t%g\n',ctf.res4.filter(i).parameters)
    end
  end
end





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% READ SENSOR INFORMATION

% seek channel information byte offset
fseek(res4fid,b+f,-1);

% read channel names (not trivial!)
for channel = 1:ctf.res4.generalSetup.numberChannels,
  temp = fread(res4fid,[1,32],'char');
  temp(temp>127) = 0;
  temp(temp<0) = 0;
  temp = strtok(temp,char(0));
  temp = strtok(temp,'-'); % this is in brainstorm code
  ctf.sensor.info(channel).label = char(temp);
  ctf.sensor.label{1,channel} = char(temp);
end


for chan = 1:ctf.res4.generalSetup.numberChannels,
  
  %ftell(res4fid);
  
  ctf.sensor.info(chan).index = fread(res4fid,1,'int16');
  ctf.sensor.info(chan).extra = fread(res4fid,1,'int16');
  id = fread(res4fid,1,'int32')+1;
  ctf.sensor.info(chan).proper_gain = fread(res4fid,1,'double');
  ctf.sensor.info(chan).q_gain = fread(res4fid,1,'double');
  ctf.sensor.info(chan).io_gain = fread(res4fid,1,'double');
  ctf.sensor.info(chan).io_offset = fread(res4fid,1,'double');
  fread(res4fid,1,'int16');
  fread(res4fid,1,'int16');
  fread(res4fid,1,'int32');
  
  %fseek(res4fid,ftell(res4fid)+6,0);
  
  for pos = 1:8,
    ctf.sensor.info(chan).coil(pos).position.x = fread(res4fid,1,'double');
    ctf.sensor.info(chan).coil(pos).position.y = fread(res4fid,1,'double');
    ctf.sensor.info(chan).coil(pos).position.z = fread(res4fid,1,'double');
    fread(res4fid,1,'double');
    ctf.sensor.info(chan).coil(pos).orient.x = fread(res4fid,1,'double');
    ctf.sensor.info(chan).coil(pos).orient.y = fread(res4fid,1,'double');
    ctf.sensor.info(chan).coil(pos).orient.z = fread(res4fid,1,'double');
    fread(res4fid,1,'double');
    fread(res4fid,1,'int16');
    fread(res4fid,1,'int32');
    fread(res4fid,1,'int16');
    fread(res4fid,1,'double');
    
    %fseek(res4fid,ftell(res4fid)+56,0);
    %fseek(res4fid,ftell(res4fid)-80,0);
  end
  
  for pos = 1:8,
    ctf.sensor.info(chan).hcoil(pos).position.x = fread(res4fid,1,'double');
    ctf.sensor.info(chan).hcoil(pos).position.y = fread(res4fid,1,'double');
    ctf.sensor.info(chan).hcoil(pos).position.z = fread(res4fid,1,'double');
    fread(res4fid,1,'double');
    ctf.sensor.info(chan).hcoil(pos).orient.x = fread(res4fid,1,'double');
    ctf.sensor.info(chan).hcoil(pos).orient.y = fread(res4fid,1,'double');
    ctf.sensor.info(chan).hcoil(pos).orient.z = fread(res4fid,1,'double');
    fread(res4fid,1,'double');
    fread(res4fid,1,'int16');
    fread(res4fid,1,'int32');
    fread(res4fid,1,'int16');
    fread(res4fid,1,'double');
    
    %fseek(res4fid,ftell(res4fid)+56,0);
    %fseek(res4fid,ftell(res4fid)+80,0);
  end
  %fseek(res4fid,ftell(res4fid)+1288,-1);
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find channel types

ctf.sensor.type.eeg = 9;
ctf.sensor.type.meg = 5;
ctf.sensor.type.ref = [0 1];


%ctf.sensor.type.vc  = ??;
%vcsens = find([ctf.sensor.info.index] == ??);


eegsens = find([ctf.sensor.info.index] == 9);
megsens = find([ctf.sensor.info.index] == 5);
refsens = find([ctf.sensor.info.index] == 1);
refsens = [refsens,find([ctf.sensor.info.index] == 0)];
allsens = [eegsens,megsens,refsens];
othersens = setdiff([1:ctf.res4.generalSetup.numberChannels],allsens);

ctf.sensor.index = struct(...
  'eeg',{eegsens},...
  'meg',{megsens},...
  'ref',{refsens},...
  'other',{othersens} ...
  );



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Location coordinates of channels in centimeters, in patient head space

for chan = 1:ctf.res4.generalSetup.numberChannels,
  
  switch ctf.sensor.info(chan).index,
    
    case {0,1,5},
      %0=Reference Channels, 
      %1=More Reference Channels, 
      %5=MEG Channels
      
      coord = [ctf.sensor.info(chan).hcoil(1:2).position];
      ctf.sensor.info(chan).location = [coord.x; coord.y; coord.z];
      
      orient = [ctf.sensor.info(chan).hcoil(1).orient];
      ctf.sensor.info(chan).orientation = [orient.x; orient.y; orient.z];
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % This ensures that the orientation of the sensor is away from the
      % center of a sphere.  It uses the sign of the dot product between
      % the orientation vector and the location vector.
      tmp = ctf.sensor.info(chan).orientation' * ctf.sensor.info(chan).location;
      tmp = sign(tmp(1));
      ctf.sensor.info(chan).orientation = tmp * ctf.sensor.info(chan).orientation;
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
    case 9,
      %EEG Channels    
      
      coord = [ctf.sensor.info(chan).hcoil(1:2).position];
      ctf.sensor.info(chan).location = [coord.x; coord.y; coord.z];
      ctf.sensor.info(chan).orientation = [];
      
  end
end








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NUMBER OF COEFFICIENTS, byte offset = b+f+nc*1360, byte size = 1

offset = b + f + ctf.res4.generalSetup.numberChannels * 1360;
fseek(res4fid,offset,-1);
ctf.res4.numberCoefficients = fread(res4fid,1,'int16');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SENSOR COEFFICIENT RECORDS, byte offset = b+f+nc*1360+2, byte size = 1992

if VERBOSE & ctf.res4.numberCoefficients,
  fprintf('...reading %d coefficients\n',ctf.res4.numberCoefficients);
end

SENSOR_LABEL  = 31;
MAX_NUM_COEFS = 50;
MAX_BALANCING = MAX_NUM_COEFS;

hexadef = {'00000000','47314252','47324252','47334252','47324f49','47334f49'};
strdef = {'NOGRAD','G1BR','G2BR','G3BR','G2OI','G3OI'};

for i = 1:ctf.res4.numberCoefficients,
  
  % read the sensor name (channel label)
  temp = fread(res4fid,[1,32],'char');
  temp(temp>127) = 0;
  temp(temp<0) = 0;
  temp = strtok(temp,char(0));
  temp = strtok(temp,'-');
  sensorName = char(temp);
  sensorIndex = strmatch( sensorName, ctf.sensor.label );
  
  % read the coefficient type
  coefType = fread(res4fid,1,'bit32');
  
  padding = fread(res4fid,1,'int32'); % not sure why this is needed???
  
  % read the coefficient record
  numberCoefs = fread(res4fid,1,'int16');
  
  if numberCoefs > MAX_NUM_COEFS,
    msg = sprintf('numberCoefs > MAX_NUM_COEFS\n');
    warning(msg);
  end
  
  sensor_list = char(fread(res4fid,[SENSOR_LABEL,MAX_BALANCING],'uchar')');
  % clean-up the sensor_list
  sensor_list = sensor_list(1:numberCoefs,:);
  for i=1:numberCoefs,
    
    temp = strtok(sensor_list(i,:),char(0));
    temp = strtok(temp,'-');
    % check if this sensor is a reference
    refLabels = ctf.sensor.label(refsens);
    refIndex = strmatch(temp,refLabels);
    if refIndex,
      % ensure this one has the same label
      temp = refLabels{refIndex};
    end
    new_sensor_list(i,:) = temp;
    
  end
  sensor_list = new_sensor_list;
  
  coefs_list = fread(res4fid,MAX_BALANCING,'double');
  % clean-up the coefs_list
  coefs_list = coefs_list(1:numberCoefs,:);
  
  % allocate the coefficient parameters into the ctf struct
  ctf.res4.sensorCoef(i).sensorName = sensorName;
  ctf.res4.sensorCoef(i).coefType   = coefType;
  ctf.res4.sensorCoef(i).coefRec.numberCoefs = numberCoefs;
  ctf.res4.sensorCoef(i).coefRec.sensor_list = sensor_list;
  ctf.res4.sensorCoef(i).coefRec.coefs_list  = coefs_list;
  
  
  
%   % DLW:
%   % This is a brainstorm variable, note the use of coefType
%   % Not clear why this is checked and allocated as such
%   % It is not returned by this function, as yet
%   coefType = find( hex2dec(hexadef) == coefType );
%   if coefType,
%     CoefInfo{sensorIndex,coefType-1}.numberCoefs = numberCoefs;
%     for i=1:numberCoefs,
%       CoefInfo{sensorIndex,coefType-1}.sensor_list(i) = sensor_list(i);
%       CoefInfo{sensorIndex,coefType-1}.coefs(i) = coefs_list(i);
%     end
%   end
  
  
  
end


t = toc; fprintf('...done (%6.2f sec)\n\n',t);

return
